#!/usr/bin/python2.6
# -*- coding: iso-8859-1 -*-

 #   This software was developed by Zanoni Dias, Ulisses Dias and Joćo
 #   C. Setubal
 #
 #   It should not be redistributed or used for any commercial purpose
 #   without written permission from authors
 #
 #   release date: nov 15, 2011
 #
 # This software is experimental in nature and is
 # supplied "AS IS", without obligation by the authors to provide
 # accompanying services or support.  The entire risk as to the quality
 # and performance of the Software is with you. The authors
 # EXPRESSLY DISCLAIM ANY AND ALL WARRANTIES REGARDING THE SOFTWARE,
 # WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO WARRANTIES
 # PERTAINING TO MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 #

import sys
import re

def parse_sis(sis_filename) :
    sis_file = open(sis_filename)
    scaffolds     = []
    line_count    = 0

    for line in sis_file :
        line_count = line_count + 1
        if line.rstrip() :
            match = re.match("^>(.*?)$", line)
            if match :
                scaffolds.append([match.group(1).rstrip().lstrip(),
                                 []])
            else :
                match = re.match("^(.*) ([01])$", line)
                if match :
                    scaffolds[-1][1].append([match.group(1),
                                            int(match.group(2))]
                                            )
                else :
                    print("Warning: ignoring line %s:\n %s" % (line_count,
                                                               line))

    return scaffolds

# Read the draft genome and create a contig dictionary
def contig_dictionary(genome_array) :
    contig_dict  = {}
    last_contig = ""
    last_count  = 0

    i = 0

    for i in range(len(genome_array)) :
        line = genome_array[i]
        match = re.match("^>(.*?)( .*)?$", line)
        if match :
            if last_contig :
                contig_dict[last_contig].append(i)
                contig_dict[last_contig].append(last_count)

            last_contig = match.group(1)
            last_count  = 0
            if not last_contig in contig_dict :
                contig_dict[last_contig] = [match.group(2), i+1]
            else :
                print("Two contigs with the same name")
                sys.exit()
        else :
            last_count = last_count + len(line.rstrip().lstrip())
    contig_dict[last_contig].append(len(genome_array))
    contig_dict[last_contig].append(last_count)
    return contig_dict

def reverse_string(input_sequence) :
    output_sequence = ""
    for el in input_sequence.rstrip() :
        new_el = ""
        if el == "A" :
            new_el = "T"
        elif el == "T" :
            new_el = "A"
        elif el == "G" :
            new_el = "C"
        elif el == "C" :
            new_el = "G"
        elif el == "N" :
            new_el = "N"
        elif el == "S" :
            new_el = "S"
        elif el == "W" :
            new_el = "W"
        elif el == "M" :
            new_el = "K"
        elif el == "K" :
            new_el = "M"
        elif el == "R" :
            new_el = "Y"
        elif el == "Y" :
            new_el = "R"
        else :
#            print("ERRO: Caracter nao permitido no genoma '%s'" % el)
            new_el = ""
        output_sequence = "%s%s" % (new_el,output_sequence)
    return "%s\n" % output_sequence


####################################################
#################### MAIN ##########################
################## FUNCTION ########################
####################################################
def main():
    ## Reading SIS file
    scaffolds = parse_sis(sys.argv[1])

    ## Reading Draft Genome
    draft       = open(sys.argv[2])
    draft_array = draft.readlines()
    contig_dict = contig_dictionary(draft_array)

    for scaffold  in scaffolds :
        scaffold_file = open("%s.fna" % scaffold[0],"w")

        for contig in scaffold[1] :
            scaffold_file.write(">%s \n" % contig[0])

            contig_range = list(range(contig_dict[contig[0]][1],
                                      contig_dict[contig[0]][2])
            )

            if contig[1] :
                contig_range.reverse()
                for j in contig_range :
                    scaffold_file.write(reverse_string(draft_array[j]))
            else :
                for j in contig_range :
                    scaffold_file.write(draft_array[j])



#!/usr/bin/python
# -*- coding: iso-8859-1 -*-

 #   This software was developed by Zanoni Dias, Ulisses Dias and Joćo
 #   C. Setubal
 #
 #   It should not be redistributed or used for any commercial purpose
 #   without written permission from authors
 #
 #   release date: nov 15, 2011
 #
 # This software is experimental in nature and is
 # supplied "AS IS", without obligation by the authors to provide
 # accompanying services or support.  The entire risk as to the quality
 # and performance of the Software is with you. The authors
 # EXPRESSLY DISCLAIM ANY AND ALL WARRANTIES REGARDING THE SOFTWARE,
 # WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO WARRANTIES
 # PERTAINING TO MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 #

import sys
import re

# Returns the second element of an array
def second_element(array) :
    return array[1]

# Returns the least element between the first and the second element
# of an array
def min_first_second_element(array) :
    return min(array[0], array[1])

# Parse the coordinate file generated by nucmer or promer
def parse_coords(coords_filename) :
    info_array  = []
    coords_file = open(coords_filename)
    coords_file.readline()
    coords_file.readline()
    coords_file.readline()
    coords_file.readline()
    coords_file.readline()
    for line in coords_file :
        match =  re.match("\s+(\S+)\s+(\S+)\s+\|\s+(\S+)\s+(\S+)\s+\|(.*)\s(\S+)", line)
        if match :
            s1     = int(match.group(1))
            e1     = int(match.group(2))
            s2     = int(match.group(3))
            e2     = int(match.group(4))
            extra  = match.group(5)
            contig = match.group(6)
            info_array.append([s1,e1,s2,e2,contig])
    coords_file.close()
    info_array.sort(key=min_first_second_element)
    return info_array

# Receives an array of coordinates and generates the contig
# representation as a set of integers. The assumption we make
# here is that the reference is the identity permutation.
def coords_to_permutation(coords) :
    contigs         = {}
    best_scored_mum = {}
    count           = 1

    def get_orientation(lcount,ls1,le1,ls2,le2) :
        if ls1 < le1 :
            if ls2 < le2 :
                return [ lcount, ls2, le2]
            else :
                return [-lcount, le2, ls2]
        else :
            if ls2 < le2 :
                return [-lcount, ls2, le2]
            else :
                return [ lcount, le2, ls2]

    for el in coords :
        s1     = el[0]
        e1     = el[1]
        s2     = el[2]
        e2     = el[3]
        contig = el[4]
        aux = get_orientation(count, s1, e1, s2, e2)
        if not contig in contigs :
            contigs[contig] = []
        contigs[contig].append([aux[0], aux[1], aux[2], contig])
        count = count + 1

    contigs_array = []
    for contig in contigs.keys() :
        contigs[contig].sort(key=second_element)
        contigs_array.append([contig])
        for el in contigs[contig] :
            contigs_array[-1].append(el[0])

    return contigs_array


##################################################################
class InContigElement:
  def __init__(self,contig,position,sign):
    self.contig = contig
    self.position = position
    self.sign = sign

##################################################################
# Return the sign of an element
def sign(n):

  return n / abs(n)

##################################################################
# Return an array of size n filled with zeros
def zeros(n):

  z = []
  for i in range(n):
    z.append(0)

  return z

##################################################################
# Create the data structure that allows the search for the contig
# where one element is placed in.
def makeInContig(contigs,c):

  inContig = []
  for i in range(n+1):
    inContig.append(None)

  for i in range(c):
    for j in range(len(contigs[i])):
      element = abs(contigs[i][j])
      inContig[element] = InContigElement(i,j,sign(contigs[i][j]))

  return inContig

##################################################################
# Generate the scaffold from a set of contigs. This procedure is the
# main core of our method.
def makeScaffold2(contigs,inContig,c,n):
  scaffold = []
  lookingfor = 1
  used = zeros(c)
  next = 0
  s = 0
  fail = 0
  ok = 0
  searched = []
  restart    = True
  breakpoint = True

  # While there is contig to place in the righ position
  while s < c:
    reset = 0

    # If we are looking for a valid element
    if lookingfor != n+1:

      element = inContig[abs(lookingfor)]

      if sign(lookingfor) == element.sign:
        if element.position != 0:
          lookingfor = contigs[element.contig][element.position-1] + 1
          breakpoint = True
        else:
          ok = 1
      else:
        if element.position != len(contigs[element.contig])-1:
          lookingfor = - (contigs[element.contig][element.position+1] - 1)
          breakpoint = True
        else:
          ok = 1

    # If we are looking for a new element
    if searched.count(lookingfor) == 0:
      searched.append(lookingfor)
    else:
      ok = 1

    if ok:
      if not(used[element.contig]):
        scaffold.append([sign(lookingfor) * (element.contig+1) * element.sign,restart,breakpoint])
        restart    = False
        breakpoint = False
        s += 1
        searched = []
        used[element.contig] = 1
        while next < c and used[next]:
          next += 1

        if sign(lookingfor) == element.sign:
          lookingfor = contigs[element.contig][-1] + 1
        else:
          lookingfor = -contigs[element.contig][0] + 1

        ok = 0
      else:
        reset = 1

    # Check if it is necessary to start a new search using a
    # non-placed contig
    if (s < c) and (lookingfor == n+1 or reset):
      lookingfor = contigs[next][0]
      restart    = True
      breakpoint = True
      searched = []
      fail += 1

  return [scaffold, fail]

#######################################################
####################### Main ##########################
#######################################################
def main():
    try :
        # Parse the input
        coords = parse_coords(sys.argv[1])

        # Generate the contig array
        contig_array = coords_to_permutation(coords)
        contigs = []
        for contig in contig_array :
            contigs.append(contig[1:])

        c = len(contigs)
        n = len(coords)

        # Generate the inContig structure. This structure make it easier
        # to search for the contig where a given element is placed in
        inContig = makeInContig(contigs,c)

        # Generate the scaffold
        scaffold = makeScaffold2(contigs, inContig, c, n)

        # Print the scaffold

        scaffold_count   = 1
        print("> Scaffold_%s" % scaffold_count)
        for el in scaffold[0] :
            if el[1] :
                if scaffold_count > 1 :
                    print("")
                    print("> Scaffold_%s" % scaffold_count)
                scaffold_count = scaffold_count + 1

            if el[0] < 0 :
                print("%s 1" % contig_array[int(abs(el[0])-1)][0])
            else :
                print("%s 0" % contig_array[int(el[0]-1)][0])
    except :
        print("Usage: %s <coord file>" % sys.argv[0])
        print("   <coord file> is the coordinate file of show-coord (MUMmer)")



"""
A setuptools based setup module.
See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

from setuptools import setup, find_packages
import re
from codecs import open
from os import path
import sys


if sys.version_info <= (3, 0):
    sys.stderr.write("ERROR: SIS requires Python 3 " +
                     "or above...exiting.\n")
    sys.exit(1)

setup(
    name='scaffoldsis',
    version="0.0.0",

    description='genome scaffolder for prokaryotes',
    long_description="""
    a program to generate draft genome sequence scaffolds for prokaryotes.
    https://bmcbioinformatics.biomedcentral.com/articles/10.1186/1471-2105-13-96
    by Zanoni Dias, Ulisses Dias and João C Setubal
    """,
    url='https://github.com/nickp60/SIS',
    author='Nick Waters',
    author_email='nickp60@gmail.com',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
    ],
    keywords='bioinformatics',
    packages=find_packages(),
    package_data={
       '': [path.join(__name__, "sis", "examples/*")],
    },
    entry_points={
       'console_scripts': [
           'sis.py=sis.SIS:main',
           'multifasta.py=sis.multifasta:main',
       ],
    },
)



# -*- coding: utf-8 -*-
"""
Created on Fri Nov 10 08:57:31 2017
@author: nicholas

"""
import sys
import logging
import shutil
import time
import os
import unittest

logger = logging

from .context import sis
from sis import sis, multifasta

@unittest.skipIf((sys.version_info[0] != 3) or (sys.version_info[1] < 5),
                 "Subprocess.call among other things wont run if tried " +
                 " with less than python 3.5")
class sisTestCase(unittest.TestCase):
    """ tests for sis.py
    """
    def setUp(self):
        self.startTime = time.time()
        self.to_be_removed = []

    def test_zeros(self):
        """ return an array filled with zeros"""
        self.assertEqual(sis.zeros(7), [0,0,0,0,0,0,0])


    def tearDown(self):
        """ delete temp files if no errors """
        for filename in self.to_be_removed:
            try:
                os.unlink(filename)
            except IsADirectoryError:
                shutil.rmtree(filename)
        t = time.time() - self.startTime
        print("%s: %.3f" % (self.id(), t))

if __name__ == '__main__':
    unittest.main()
